from django.core.management import call_command

from arkindex.documents.models import (
    Classification,
    Element,
    ElementType,
    Entity,
    MetaData,
    MetaType,
    Transcription,
    TranscriptionEntity,
)
from arkindex.process.models import WorkerRun
from arkindex.project.tests import FixtureTestCase


class TestCacheWorkerRuns(FixtureTestCase):

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.run_1, cls.run_2, cls.run_3, cls.run_4, cls.run_5 = WorkerRun.objects.all().order_by("summary")
        # Set has_results to True for run_3 and run_4
        cls.run_3.has_results = True
        cls.run_4.has_results = True
        cls.run_3.save()
        cls.run_4.save()
        # Remove all Element, Classifications, MetaData, Entity, TranscriptionEntity and Transcription objects
        # to start clean and be sure of which runs have and do not have ml results
        for model in [TranscriptionEntity, MetaData, Entity, Transcription, Classification, Element]:
            model.objects.all().delete()
        # Create objects created by run_2 and run_4
        page = Element.objects.create(
            corpus=cls.corpus,
            type=ElementType.objects.get(slug="page"),
            name="Fox Mulder",
            worker_run=cls.run_2,
            worker_version=cls.run_2.version
        )
        MetaData.objects.create(
            name="partner",
            type=MetaType.Text,
            value="Dana Scully",
            element=page,
            worker_run=cls.run_4,
            worker_version=cls.run_4.version
        )

    def test_run(self):
        self.assertQuerysetEqual(
            WorkerRun.objects.order_by("summary").values_list("id", "has_results"),
            [(self.run_1.id, False), (self.run_2.id, False), (self.run_3.id, True), (self.run_4.id, True), (self.run_5.id, False)]
        )
        with self.assertNumQueries(2):
            call_command("cache_worker_runs")
        self.assertQuerysetEqual(
            WorkerRun.objects.order_by("summary").values_list("id", "has_results"),
            [(self.run_1.id, False), (self.run_2.id, True), (self.run_3.id, False), (self.run_4.id, True), (self.run_5.id, False)]
        )
